/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER, TCSDESIGNER
 * Version: 1.0
 */

using NAnt.Core;
using NAnt.Core.Types;
using System;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// <para>
    /// This class is an abstract base class for all the classes representing a server to be used when running tests.
    /// It stores some attributes passed from the <see cref="ServerElement"/> class. The appropriate properties are
    /// provided for accessing the attributes.
    /// </para>
    /// <para>
    /// It also has properties for accessing the nested "sourcefiles" and "credentials" elements of the "server" XML
    /// element. The other important thing is a property for specifying the options of the server.
    /// </para>
    /// <para>
    /// The class provides the abstract methods of starting/stopping/initializing data/clean-up, which should be
    /// implemented by the derived classes.
    /// </para>
    /// </summary>
    /// <remarks>
    /// This class is mutable, i.e. not thread-safe.
    /// </remarks>
    public abstract class Server
    {
        /// <summary>
        /// This variable represents the value of the "class" attribute. This attribute represents the name of class
        /// derived from <see cref="Server"/>, which represents the server described by this element.
        /// The attribute is required. Initial value is null. It can be accessed via "ClassName" property.
        /// </summary>
        private string className = null;

        /// <summary>
        /// This variable represents the "startserver" attribute. This attribute specifies whether the server should
        /// be started by the component when running tests. This attribute is not required, the default value is true.
        /// Initial value is true. Can be accessed via the <see cref="StartServer"/> property.
        /// </summary>
        private bool startServer = true;

        /// <summary>
        /// This variable represents the "initserver" attribute. This specifies whether the server should be
        /// initialized with data by the component when running tests. This attribute is not required, the default
        /// value is true. Initial value is true. Can be accessed via the <see cref="InitServer"/> property.
        /// </summary>
        private bool initServer = true;

        /// <summary>
        /// <para>
        /// This variable represents the collection of nested "option" XML elements. These represent the options of the
        /// server. The options can be any key-value pairs, where keys and values are strings.
        /// </para>
        /// <para>
        /// The value specified by the "class" attribute, a class derived from <see cref="Server"/>, may interpret the
        /// options in any relevant manner. This nested element can possibly be omitted if the specified server doesn't
        /// require it.
        /// </para>
        /// <para>
        /// Can be accessed via the <see cref="Options"/> property.
        /// </para>
        /// </summary>
        private OptionCollection options = null;

        /// <summary>
        /// <para>
        /// This variable represents the nested "credentials" XML element, which represents the credentials used to
        /// authenticate to the server.
        /// </para>
        /// <para>
        /// This includes user name, password and domain. This nested element can possibly be omitted, it depends on
        /// the particular server.
        /// </para>
        /// <para>
        /// Can be accessed via the <see cref="Credentials"/> property.
        /// </para>
        /// </summary>
        private Credential credentials = null;

        /// <summary>
        /// <para>
        /// This variable represents the nested "sourcefiles" XML element, which represents the files used when
        /// initializing the server with data.
        /// </para>
        /// <para>
        /// This can be an application to deploy for an application server or some SQL scripts, DB tables, etc. for a
        /// data server, or some other files the particular server may need for initialization. This nested element can
        /// possibly be omitted if the specified by "class" attribute class, derived from <see cref="Server"/>, permits
        /// it to be omitted.
        /// </para>
        /// <para>
        /// Can be accessed via the <see cref="SourceFiles"/> property.
        /// </para>
        /// </summary>
        private FileSet sourceFiles = null;

        /// <summary>
        /// This variable is used by subclasses (through the <see cref="Project"/> property) to retrieve the project
        /// the task using this server belongs to. It can be used for constructing subtasks with a proper project, or
        /// other operations that depend on project data.
        /// </summary>
        private Project project = null;

        /// <summary>
        /// This property represents the value of the "class" attribute. This attribute represents the name of class
        /// derived from <see cref="Server"/>, which represents the server described by this element.
        /// The attribute is required.
        /// </summary>
        /// <value>
        /// Any string, initially null. The setter performs no verification.
        /// </value>
        public string ClassName
        {
            get
            {
                return className;
            }
            set
            {
                className = value;
            }
        }

        /// <summary>
        /// This property represents the "startserver" attribute. This attribute specifies whether the server should
        /// be started by the component when running tests. This attribute is not required, the default value is true.
        /// </summary>
        /// <value>
        /// A bool, initially true.
        /// </value>
        public bool StartServer
        {
            get
            {
                return startServer;
            }
            set
            {
                startServer = value;
            }
        }

        /// <summary>
        /// This property represents the "initserver" attribute. This specifies whether the server should be
        /// initialized with data by the component when running tests. This attribute is not required, the default
        /// value is true.
        /// </summary>
        /// <value>
        /// A bool, initially true.
        /// </value>
        public bool InitServer
        {
            get
            {
                return initServer;
            }
            set
            {
                initServer = value;
            }
        }

        /// <summary>
        /// <para>
        /// This property represents the collection of nested "option" XML elements. These represent the options of the
        /// server. The options can be any key-value pairs, where keys and values are strings.
        /// </para>
        /// <para>
        /// The string constants representing the names of the most common options are provided by the class. They are:
        /// <see cref="TIMEOUT"/>, <see cref="PORT"/> and <see cref="ADDRESS"/>. The derived classes should provide
        /// similar string constants representing the names of options they need.
        /// </para>
        /// </summary>
        /// <value>
        /// A non-null, never changing <see cref="OptionCollection"/> reference. The content is to be set by
        /// <see cref="ServerElement"/> or another client.
        /// </value>
        public OptionCollection Options
        {
            get
            {
                return options;
            }
            set
            {
                options = value;
            }
        }

        /// <summary>
        /// <para>
        /// This property represents the nested "credentials" XML element, which represents the credentials used to
        /// authenticate to the server.
        /// </para>
        /// <para>
        /// This includes user name, password and domain. This nested element can possibly be omitted, it depends on
        /// the particular server.
        /// </para>
        /// </summary>
        /// <value>
        /// A non-null, never changing, <see cref="Credential"/> reference. The content is to be set by
        /// <see cref="ServerElement"/> or another client.
        /// </value>
        public Credential Credentials
        {
            get
            {
                return credentials;
            }
            set
            {
                credentials = value;
            }
        }

        /// <summary>
        /// <para>
        /// This property represents the nested "sourcefiles" XML element, which represents the files used when
        /// initializing the server with data.
        /// </para>
        /// <para>
        /// This can be an application to deploy for an application server or some SQL scripts, DB tables, etc. for a
        /// data server, or some other files the particular server may need for initialization. This nested element can
        /// possibly be omitted if the specified by "class" attribute class, derived from <see cref="Server"/>, permits
        /// it to be omitted.
        /// </para>
        /// </summary>
        /// <value>
        /// A non-null, never changing, <see cref="FileSet"/> reference. The content is to be set by
        /// <see cref="ServerElement"/> or another client.
        /// </value>
        public FileSet SourceFiles
        {
            get
            {
                return sourceFiles;
            }
            set
            {
                sourceFiles = value;
            }
        }

        /// <summary>
        /// <para>
        /// This string constant represents the name (key) of the option that the derived classes should use for
        /// getting the timeout, which is a time in milliseconds, the <see cref="StartUp"/> method should wait for the
        /// server to start.
        /// </para>
        /// <para>
        /// If the specified time passes and the server is still not running, the <see cref="StartUp"/> method should
        /// throw an exception.
        /// </para>
        /// <para>
        /// The behavior described here is only a suggestion, the actual subclasses can treat options in any way they
        /// like.
        /// </para>
        /// </summary>
        public const string TIMEOUT = "timeout";

        /// <summary>
        /// <para>
        /// This string constant represents the name (key) of the option that the derived classes should use for
        /// getting the number of the port used when establishing the server connection.
        /// </para>
        /// <para>
        /// The behavior described here is only a suggestion, the actual subclasses can treat options in any way they
        /// like.
        /// </para>
        /// </summary>
        public const string PORT = "port";

        /// <summary>
        /// <para>
        /// This string constant represents the name (key) of the option that the derived classes should use for
        /// getting the address of machine running the server.
        /// </para>
        /// <para>
        /// The behavior described here is only a suggestion, the actual subclasses can treat options in any way they
        /// like.
        /// </para>
        /// </summary>
        public const string ADDRESS = "address";

        /// <summary>
        /// <para>
        /// This property represents the value of the option that the derived classes should use for determining the
        /// timeout, which is an integral number of milliseconds. The <see cref="StartUp"/> method should wait for the
        /// server to start.
        /// </para>
        /// <para>
        /// If the specified time passes and the server is still not running, the <see cref="StartUp"/> method should
        /// throw an exception.
        /// </para>
        /// <para>
        /// The behavior described here is only a suggestion, the actual subclasses can treat options in any way they
        /// like.
        /// </para>
        /// </summary>
        /// <exception cref="ServerException">Any error while retrieving the option value, including a value that is
        /// not a non-negative integer.</exception>
        /// <value>
        /// A non-negative integer, indicating the timeout in ms.
        /// </value>
        public int Timeout
        {
            get
            {
                return GetNonNegativeOption(TIMEOUT);
            }
        }

        /// <summary>
        /// <para>
        /// This property represents the value of the option that the derived classes should use as the port used when
        /// establishing the server connection.
        /// </para>
        /// <para>
        /// The behavior described here is only a suggestion, the actual subclasses can treat options in any way they
        /// like.
        /// </para>
        /// </summary>
        /// <exception cref="ServerException">Any error while retrieving the option value, including a value that is
        /// not a non-negative integer.</exception>
        /// <value>
        /// A non-negative integer, indicating the port number of the connection.
        /// </value>
        public int Port
        {
            get
            {
                return GetNonNegativeOption(PORT);
            }
        }

        /// <summary>
        /// This property is used by subclasses to retrieve the project the task using this server belongs to. It can
        /// be used for constructing subtasks with a proper project, or other operations that depend on project data.
        /// </summary>
        /// <value>
        /// An instance of <see cref="Project"/>, preferably not null, as that may cause subclasses to fail.
        /// </value>
        public Project Project
        {
            get
            {
                return project;
            }
            set
            {
                project = value;
            }
        }

        /// <summary>
        /// <para>
        /// This property represents the value of the option that the derived classes should use as address of machine
        /// running the server.
        /// </para>
        /// <para>
        /// The behavior described here is only a suggestion, the actual subclasses can treat options in any way they
        /// like.
        /// </para>
        /// </summary>
        /// <exception cref="ServerException">Any error while retrieving the option value.</exception>
        /// <value>
        /// A string indicating the address for the connection.
        /// </value>
        public string Address
        {
            get
            {
                return GetOption(ADDRESS);
            }
        }

        /// <summary>
        /// Retrieves a specific option value.
        /// </summary>
        /// <param name="name">The name of the option to check.</param>
        /// <exception cref="ServerException">Any error while retrieving the option value, or if it is not a
        /// non-negative integer.</exception>
        /// <returns>A non-null string, the option value.</returns>
        private int GetNonNegativeOption(string name)
        {
            try
            {
                int value = Int32.Parse(GetOption(name));
                if (value < 0)
                {
                    throw new ServerException("Negative value not allowed.");
                }

                return value;
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException("Error retrieving value.", e);
            }
        }

        /// <summary>
        /// Retrieves a specific option value, this helper is <c>protected internal</c> to handle
        /// <see cref="ApplicationServer"/> and <see cref="DatabaseServer"/>.
        /// </summary>
        /// <param name="name">The name of the option to check.</param>
        /// <exception cref="ServerException">Any error while retrieving the option value.</exception>
        /// <returns>A non-null string, the option value.</returns>
        protected internal string GetOption(string name)
        {
            try
            {
                if (options == null)
                {
                    throw new ServerException("No options present.");
                }

                Option option = Options[name];
                if (option == null)
                {
                    throw new ServerException(String.Format("Option missing: {0}", name));
                }
                if (option.Value == null)
                {
                    throw new ServerException(String.Format("Option value missing: {0}", name));
                }

                return option.Value;
            }
            catch (ServerException)
            {
                throw;
            }
            catch (Exception e)
            {
                throw new ServerException(String.Format("Error retrieving option value: {0}", name), e);
            }
        }

        /// <summary>
        /// Protected constructor for subclasses to call.
        /// </summary>
        protected Server()
        {
        }

        /// <summary>
        /// Starts the server.
        /// </summary>
        /// <remarks>
        /// <para>
        /// This method is called by <see cref="ServerAppTestTask"/> if the "startserver" attribute is not specified to
        /// false (the default value, if missing, is true).
        /// </para>
        /// <para>
        /// Subclasses must implement this method to perform the actual task of starting up the server.
        /// </para>
        /// </remarks>
        public abstract void StartUp();

        /// <summary>
        /// Shuts the server down.
        /// </summary>
        /// <remarks>
        /// <para>
        /// This method is called by <see cref="ServerAppTestTask"/> if the "startserver" attribute is not specified to
        /// false (the default value, if missing, is true). (After the actual tests have run.)
        /// </para>
        /// <para>
        /// Subclasses must implement this method to perform the actual task of stopping the server again.
        /// </para>
        /// </remarks>
        public abstract void ShutDown();

        /// <summary>
        /// Initializes the server with data.
        /// </summary>
        /// /// <remarks>
        /// <para>
        /// This method is called by <see cref="ServerAppTestTask"/> if the "initserver" attribute is not specified to
        /// false (the default value, if missing, is true).
        /// </para>
        /// <para>
        /// Subclasses must implement this method to perform the actual task of initializing the server with data. The
        /// method will be called after <see cref="StartUp"/>, if that method is called at all.
        /// </para>
        /// </remarks>
        public abstract void InitData();

        /// <summary>
        /// Clean-ups the data initialized by the <see cref="InitData"/> method.
        /// </summary>
        /// /// <remarks>
        /// <para>
        /// This method is called by <see cref="ServerAppTestTask"/> if the "initserver" attribute is not specified to
        /// false (the default value, if missing, is true). (After the actual tests have run.)
        /// </para>
        /// <para>
        /// Subclasses must implement this method to perform the actual task of cleaning up the server data, after the
        /// tests are concluded. The method will be called before <see cref="ShutDown"/>, if that method is called at
        /// all.
        /// </para>
        /// </remarks>
        public abstract void CleanUpData();
    }
}
