/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 *
 * Author: TCSDEVELOPER
 * Version: 1.0
 */

using NAnt.Core;
using NAnt.Core.Attributes;
using NUnit.Framework;
using System;
using System.IO;
using System.Reflection;
using System.ServiceProcess;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// This class tests <see cref="ServerAppTestTask"/>, including proper attribute markup, and real tests with the
    /// NAnt build framework.
    /// </summary>
    [TestFixture]
    public class TestServerAppTestTask
    {
        /// <summary>
        /// A file used to tell the internal tests whether they actually should run at this time. This is the simplest
        /// solution, when we don't have real NUnit 2.2 "categories".
        /// </summary>
        public const string HINTFILE = @"..\..\test_files\hint";

        /// <summary>
        /// This class is used in conjunction with <see cref="TestSimpleTask"/>.
        /// </summary>
        [TestFixture]
        public class Simple
        {
            /// <summary>
            /// The only test performed.
            /// </summary>
            [Test]
            public void OnlyTest()
            {
                if (File.Exists(HINTFILE))
                {
                    ServiceController controller = new ServiceController("w3svc");
                    Assert.AreEqual(ServiceControllerStatus.Running, controller.Status,
                        "The service should be started.");
                }
            }
        }

        /// <summary>
        /// This class is used in conjunction with <see cref="TestFailingTask"/>.
        /// </summary>
        [TestFixture]
        public class Failing
        {
            /// <summary>
            /// The only test performed, will always fail, when the hint file is present.
            /// </summary>
            [Test]
            public void OnlyTest()
            {
                if (File.Exists(HINTFILE))
                {
                    int x = 1;
                    int y = 0;

                    x /= y;
                }
            }
        }

        /// <summary>
        /// The object used for testing.
        /// </summary>
        private ServerAppTestTask testInstance;

        /// <summary>
        /// Sets up the testing fixture
        /// </summary>
        [SetUp]
        public void Setup()
        {
            testInstance = new ServerAppTestTask();
            using (File.Create(HINTFILE))
            {
            }
        }

        /// <summary>
        /// Remove the "hint file".
        /// </summary>
        [TearDown]
        public void Teardown()
        {
            File.Delete(HINTFILE);
        }

        /// <summary>
        /// Tests the servers property, reading default, setting.
        /// </summary>
        [Test]
        public void TestServers()
        {
            Assert.IsNull(testInstance.Servers, "The server array should be null by default.");

            ServerElement[] servers = new ServerElement[] { new ServerElement() };
            testInstance.Servers = servers;

            Assert.AreEqual(servers, testInstance.Servers, "The servers property was not properly set.");
        }

        /// <summary>
        /// Verifies the class attribute.
        /// </summary>
        [Test]
        public void TestClassAttributes()
        {
            Type type = typeof(ServerAppTestTask);
            object[] attrs = type.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One class attribute should be present at this level.");

            TaskNameAttribute name = (TaskNameAttribute) attrs[0];
            Assert.AreEqual("serverapptest", name.Name, "The attribute name should match.");
        }

        /// <summary>
        /// Verifies the attributes for the servers property.
        /// </summary>
        [Test]
        public void TestServersAttributes()
        {
            MemberInfo info = typeof(ServerAppTestTask).GetProperty("Servers");
            object[] attrs = info.GetCustomAttributes(false);
            Assert.AreEqual(1, attrs.Length, "One attribute should be present at this level.");

            BuildElementArrayAttribute attribute = (BuildElementArrayAttribute) attrs[0];
            Assert.AreEqual("server", attribute.Name, "The attribute name should match.");
            Assert.AreEqual(typeof(ServerElement), attribute.ElementType, "The attribute element type should match.");
            Assert.AreEqual(true, attribute.Required, "Incorrect requiredness for attribute.");
        }

        /// <summary>
        /// Tests executing a simple test, the IIS server is verified to be active within the test, and stopped
        /// afterwards.
        /// </summary>
        [Test]
        public void TestSimpleTask()
        {
            new Project(@"..\..\test_files\simple.build", Level.Info).Execute();

            ServiceController controller = new ServiceController("w3svc");
            Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status,
                "The service should be stopped again.");
        }

        /// <summary>
        /// Runs a test that will fail. Still, the server should be stopped again.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerTaskException))]
        public void TestFailingTask()
        {
            try
            {
                new Project(@"..\..\test_files\failingtask.build", Level.Info).Execute();
            }
            finally
            {
                ServiceController controller = new ServiceController("w3svc");
                Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status,
                    "The service should be stopped again.");
            }
        }

        /// <summary>
        /// Tests a configuration with two servers. Despite this, the web server should be properly stopped when the
        /// tests are finished.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ServerTaskException))]
        public void TestFailingServer()
        {
            try
            {
                new Project(@"..\..\test_files\failingserver.build", Level.Info).Execute();
            }
            finally
            {
                ServiceController controller = new ServiceController("w3svc");
                Assert.AreEqual(ServiceControllerStatus.Stopped, controller.Status,
                    "The service should be stopped again.");
            }
        }
    }
}
